<?php
/*
Plugin Name: Visitor Counter
Description: Counts unique visitors based on their IP address for a page or post and displays the number using a shortcode. Includes admin panel to configure options and styles.
Version: 1.7
Author: SB
*/

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

// Increment view count, exclude admin if the option is enabled
function vc_increment_post_view_count() {
    if ( is_single() || is_page() ) {
        global $post;
        $post_id = $post->ID;
        $user_ip = vc_get_user_ip();

        // Check if the visitor's IP is stored for this post
        $visited_ips = get_post_meta( $post_id, '_vc_visited_ips', true );
        $visited_ips = is_array( $visited_ips ) ? $visited_ips : [];

        // Check if admin should be excluded
        $exclude_admin = get_option( 'vc_exclude_admin_views', false );
        
        if ( $exclude_admin && current_user_can( 'administrator' ) ) {
            return;
        }

        // If IP not recorded, count this visit
        if ( ! in_array( $user_ip, $visited_ips ) ) {
            $visited_ips[] = $user_ip;

            // Update the post meta with the new IP
            update_post_meta( $post_id, '_vc_visited_ips', $visited_ips );

            // Increment view count
            $views = get_post_meta( $post_id, '_vc_view_count', true );
            $views = (empty($views)) ? 1 : intval($views) + 1;

            // Update the view count
            update_post_meta( $post_id, '_vc_view_count', $views );
        }
    }
}
add_action( 'wp_head', 'vc_increment_post_view_count' );

// Function to get user IP address
function vc_get_user_ip() {
    if ( !empty($_SERVER['HTTP_CLIENT_IP']) ) {
        $ip = $_SERVER['HTTP_CLIENT_IP'];
    } elseif ( !empty($_SERVER['HTTP_X_FORWARDED_FOR']) ) {
        $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    } else {
        $ip = $_SERVER['REMOTE_ADDR'];
    }
    return sanitize_text_field( $ip );
}

// Display view count via shortcode with inline CSS
function vc_display_view_count( $atts ) {
    global $post;
    $post_id = $post->ID;

    // Get the current view count
    $views = get_post_meta( $post_id, '_vc_view_count', true );

    // Default to 0 if no views yet
    if ( empty( $views ) ) {
        $views = 0;
    }

    // Define inline styles
    $label_style = 'font-weight: normal; font-size: 14px;';
    $count_style = 'color: #FF4500; font-weight: bold; font-size: 14px;';

    // Return the view count with separate classes for "Views:" and the count
    return '<span class="vc-view-label" style="' . esc_attr($label_style) . '">Views:</span> <span class="vc-view-count" style="' . esc_attr($count_style) . '">' . esc_html($views) . '</span>';
}
add_shortcode( 'visitor_count', 'vc_display_view_count' );

// Admin menu setup
function vc_add_admin_menu() {
    add_options_page(
        'Visitor Counter Settings',
        'Visitor Counter',
        'manage_options',
        'visitor-counter',
        'vc_settings_page'
    );
}
add_action( 'admin_menu', 'vc_add_admin_menu' );

// Register settings
function vc_register_settings() {
    register_setting( 'vc_settings_group', 'vc_exclude_admin_views' );
}
add_action( 'admin_init', 'vc_register_settings' );

// Admin settings page HTML
function vc_settings_page() {
    ?>
    <div class="wrap">
        <h1>Visitor Counter Settings</h1>
        <form method="post" action="options.php">
            <?php
            settings_fields( 'vc_settings_group' );
            do_settings_sections( 'vc_settings_group' );
            ?>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">Exclude Admin Views</th>
                    <td>
                        <input type="checkbox" name="vc_exclude_admin_views" value="1" <?php checked( get_option( 'vc_exclude_admin_views' ), 1 ); ?> />
                        <label for="vc_exclude_admin_views">Check this box to exclude views from administrators.</label>
                    </td>
                </tr>
            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}
